"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.RemeshScope = exports.RemeshDomainHolderContainer = exports.useRemeshDomain = exports.useRemeshSend = exports.useRemeshEvent = exports.useRemeshQuery = exports.RemeshRoot = exports.useRemeshStore = exports.useRemeshReactContext = exports.RemeshReactContext = void 0;
var tslib_1 = require("tslib");
var jsx_runtime_1 = require("react/jsx-runtime");
var react_1 = require("react");
var shim_1 = require("use-sync-external-store/shim");
var remesh_1 = require("remesh");
var useIsomorphicLayoutEffect = 
// tslint:disable-next-line: strict-type-predicates
typeof window !== 'undefined' &&
    // tslint:disable-next-line: strict-type-predicates
    typeof window.document !== 'undefined' &&
    // tslint:disable-next-line: deprecation & strict-type-predicates
    typeof window.document.createElement !== 'undefined'
    ? react_1.useLayoutEffect
    : react_1.useEffect;
exports.RemeshReactContext = (0, react_1.createContext)(null);
if (process.env.NODE_ENV !== 'production') {
    exports.RemeshReactContext.displayName = 'Remesh';
}
var useRemeshReactContext = function () {
    var context = (0, react_1.useContext)(exports.RemeshReactContext);
    if (context === null) {
        throw new Error("You may forget to add <RemeshRoot />");
    }
    return context;
};
exports.useRemeshReactContext = useRemeshReactContext;
var useRemeshStore = function () {
    var context = (0, exports.useRemeshReactContext)();
    return context.remeshStore;
};
exports.useRemeshStore = useRemeshStore;
var RemeshRoot = function (props) {
    var storeRef = (0, react_1.useRef)(props.store);
    if (!storeRef.current) {
        storeRef.current = (0, remesh_1.RemeshStore)('options' in props ? props.options : {});
    }
    var store = storeRef.current;
    var contextValue = (0, react_1.useMemo)(function () {
        return {
            remeshStore: store,
        };
    }, [store]);
    // @ts-ignore unexpected type error TS2786: 'RemeshReactContext.Provider' cannot be used as a JSX component.
    return (0, jsx_runtime_1.jsx)(exports.RemeshReactContext.Provider, tslib_1.__assign({ value: contextValue }, { children: props.children }));
};
exports.RemeshRoot = RemeshRoot;
var useRemeshQuery = function (queryAction) {
    var store = (0, exports.useRemeshStore)();
    var triggerRef = (0, react_1.useRef)(null);
    var subscribe = (0, react_1.useCallback)(function (triggerUpdate) {
        triggerRef.current = triggerUpdate;
        return function () {
            triggerRef.current = null;
        };
    }, []);
    var getSnapshot = (0, react_1.useCallback)(function () {
        var snapshot = store.query(queryAction);
        return snapshot;
    }, [store, queryAction]);
    var state = (0, shim_1.useSyncExternalStore)(subscribe, getSnapshot, getSnapshot);
    var subscriptionRef = (0, react_1.useRef)(null);
    var queryKey = store.getKey(queryAction);
    useIsomorphicLayoutEffect(function () {
        return function () {
            var _a;
            (_a = subscriptionRef.current) === null || _a === void 0 ? void 0 : _a.unsubscribe();
            subscriptionRef.current = null;
        };
    }, [store, queryKey]);
    useIsomorphicLayoutEffect(function () {
        if (subscriptionRef.current !== null) {
            return;
        }
        subscriptionRef.current = store.subscribeQuery(queryAction, function () {
            var _a;
            (_a = triggerRef.current) === null || _a === void 0 ? void 0 : _a.call(triggerRef);
        });
    }, [store, queryAction]);
    return state;
};
exports.useRemeshQuery = useRemeshQuery;
var useRemeshEvent = function (Event, callback) {
    var store = (0, exports.useRemeshStore)();
    var callbackRef = (0, react_1.useRef)(callback);
    useIsomorphicLayoutEffect(function () {
        callbackRef.current = callback;
    });
    useIsomorphicLayoutEffect(function () {
        var subscription = store.subscribeEvent(Event, function (data) {
            callbackRef.current(data);
        });
        return function () {
            subscription.unsubscribe();
        };
    }, [Event, store]);
};
exports.useRemeshEvent = useRemeshEvent;
var useRemeshSend = function () {
    var store = (0, exports.useRemeshStore)();
    return store.send;
};
exports.useRemeshSend = useRemeshSend;
var useRemeshDomain = function (domainAction) {
    var store = (0, exports.useRemeshStore)();
    var domain = store.getDomain(domainAction);
    useIsomorphicLayoutEffect(function () {
        store.igniteDomain(domainAction);
    }, [store, domainAction]);
    var domainKey = store.getKey(domainAction);
    var subscriptionRef = (0, react_1.useRef)(null);
    useIsomorphicLayoutEffect(function () {
        return function () {
            var _a;
            (_a = subscriptionRef.current) === null || _a === void 0 ? void 0 : _a.unsubscribe();
            subscriptionRef.current = null;
        };
    }, [store, domainKey]);
    useIsomorphicLayoutEffect(function () {
        if (subscriptionRef.current !== null) {
            return;
        }
        subscriptionRef.current = store.subscribeDomain(domainAction);
    }, [store, domainAction]);
    return domain;
};
exports.useRemeshDomain = useRemeshDomain;
var RemeshDomainHolder = function (props) {
    (0, exports.useRemeshDomain)(props.domain);
    return (0, jsx_runtime_1.jsx)(jsx_runtime_1.Fragment, {});
};
var RemeshDomainHolderContainer = function (props) {
    return ((0, jsx_runtime_1.jsx)(jsx_runtime_1.Fragment, { children: props.domains.map(function (domain, index) {
            return (0, jsx_runtime_1.jsx)(RemeshDomainHolder, { domain: domain }, index);
        }) }));
};
exports.RemeshDomainHolderContainer = RemeshDomainHolderContainer;
var RemeshScope = function (props) {
    return ((0, jsx_runtime_1.jsxs)(jsx_runtime_1.Fragment, { children: [(0, jsx_runtime_1.jsx)(exports.RemeshDomainHolderContainer, { domains: props.domains }), props.children] }));
};
exports.RemeshScope = RemeshScope;
//# sourceMappingURL=remesh-react.js.map