import { jsx as _jsx, Fragment as _Fragment, jsxs as _jsxs } from "react/jsx-runtime";
import { useLayoutEffect, useRef, useContext, createContext, useCallback, useMemo, useEffect } from "react";
import { useSyncExternalStore } from "use-sync-external-store/shim";
import { RemeshStore, } from "remesh";
const useIsomorphicLayoutEffect = 
// tslint:disable-next-line: strict-type-predicates
typeof window !== 'undefined' &&
    // tslint:disable-next-line: strict-type-predicates
    typeof window.document !== 'undefined' &&
    // tslint:disable-next-line: deprecation & strict-type-predicates
    typeof window.document.createElement !== 'undefined'
    ? useLayoutEffect
    : useEffect;
export const RemeshReactContext = createContext(null);
if (process.env.NODE_ENV !== 'production') {
    RemeshReactContext.displayName = 'Remesh';
}
export const useRemeshReactContext = () => {
    const context = useContext(RemeshReactContext);
    if (context === null) {
        throw new Error(`You may forget to add <RemeshRoot />`);
    }
    return context;
};
export const useRemeshStore = () => {
    const context = useRemeshReactContext();
    return context.remeshStore;
};
export const RemeshRoot = (props) => {
    const storeRef = useRef(props.store);
    if (!storeRef.current) {
        storeRef.current = RemeshStore('options' in props ? props.options : {});
    }
    const store = storeRef.current;
    const contextValue = useMemo(() => {
        return {
            remeshStore: store,
        };
    }, [store]);
    // @ts-ignore unexpected type error TS2786: 'RemeshReactContext.Provider' cannot be used as a JSX component.
    return _jsx(RemeshReactContext.Provider, { value: contextValue, children: props.children });
};
export const useRemeshQuery = function (queryAction) {
    const store = useRemeshStore();
    const triggerRef = useRef(null);
    const subscribe = useCallback((triggerUpdate) => {
        triggerRef.current = triggerUpdate;
        return () => {
            triggerRef.current = null;
        };
    }, []);
    const getSnapshot = useCallback(() => {
        const snapshot = store.query(queryAction);
        return snapshot;
    }, [store, queryAction]);
    const state = useSyncExternalStore(subscribe, getSnapshot, getSnapshot);
    const subscriptionRef = useRef(null);
    const queryKey = store.getKey(queryAction);
    useIsomorphicLayoutEffect(() => {
        return () => {
            subscriptionRef.current?.unsubscribe();
            subscriptionRef.current = null;
        };
    }, [store, queryKey]);
    useIsomorphicLayoutEffect(() => {
        if (subscriptionRef.current !== null) {
            return;
        }
        subscriptionRef.current = store.subscribeQuery(queryAction, () => {
            triggerRef.current?.();
        });
    }, [store, queryAction]);
    return state;
};
export const useRemeshEvent = function (Event, callback) {
    const store = useRemeshStore();
    const callbackRef = useRef(callback);
    useIsomorphicLayoutEffect(() => {
        callbackRef.current = callback;
    });
    useIsomorphicLayoutEffect(() => {
        const subscription = store.subscribeEvent(Event, (data) => {
            callbackRef.current(data);
        });
        return () => {
            subscription.unsubscribe();
        };
    }, [Event, store]);
};
export const useRemeshSend = function () {
    const store = useRemeshStore();
    return store.send;
};
export const useRemeshDomain = function (domainAction) {
    const store = useRemeshStore();
    const domain = store.getDomain(domainAction);
    useIsomorphicLayoutEffect(() => {
        store.igniteDomain(domainAction);
    }, [store, domainAction]);
    const domainKey = store.getKey(domainAction);
    const subscriptionRef = useRef(null);
    useIsomorphicLayoutEffect(() => {
        return () => {
            subscriptionRef.current?.unsubscribe();
            subscriptionRef.current = null;
        };
    }, [store, domainKey]);
    useIsomorphicLayoutEffect(() => {
        if (subscriptionRef.current !== null) {
            return;
        }
        subscriptionRef.current = store.subscribeDomain(domainAction);
    }, [store, domainAction]);
    return domain;
};
const RemeshDomainHolder = (props) => {
    useRemeshDomain(props.domain);
    return _jsx(_Fragment, {});
};
export const RemeshDomainHolderContainer = (props) => {
    return (_jsx(_Fragment, { children: props.domains.map((domain, index) => {
            return _jsx(RemeshDomainHolder, { domain: domain }, index);
        }) }));
};
export const RemeshScope = (props) => {
    return (_jsxs(_Fragment, { children: [_jsx(RemeshDomainHolderContainer, { domains: props.domains }), props.children] }));
};
//# sourceMappingURL=remesh-react.js.map